@@ -92,10 +92,14 @@ protected:
   bool process_Ping(cRequestPacket &r);
   bool process_GetSetup(cRequestPacket &r);
   bool process_StoreSetup(cRequestPacket &r);
+  bool process_GetSocket(cRequestPacket &r);
+  bool process_InvalidateSocket(cRequestPacket &r);
 
   bool processChannelStream_Open(cRequestPacket &r);
   bool processChannelStream_Close(cRequestPacket &req);
   bool processChannelStream_Seek(cRequestPacket &r);
+  bool processChannelStream_StatusSocket(cRequestPacket &r);
+  bool processChannelStream_StatusRequest(cRequestPacket &r);
 
   bool processRecStream_Open(cRequestPacket &r);
   bool processRecStream_Close(cRequestPacket &r);
@@ -150,7 +154,7 @@ protected:
   bool processSCAN_Start(cRequestPacket &r);
   bool processSCAN_Stop(cRequestPacket &r);
 
-  bool Undelete(cRecording* recording);
+  bool Undelete(cRecording* recording, cRecordings* reclist, cRecordings* dellist);
 
   bool processOSD_Connect(cRequestPacket &req);
   bool processOSD_Disconnect();
